#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <dir.h>
#include "wormgen.h"


#define sqr(x)      ((x)*(x))
#define random(num) ((short)((long)rand()*(num)/(RAND_MAX+1)))
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))



/*
 *  the following function generates a random x,y position on your map
 *  change this function according to your galaxy
 *
 *  This example is for a circlular map, 1000 ly radius around 2000,2000
 *
 *  the standard galaxy would be
 *  *x=1000+random(2000);
 *  *y=1000+random(2000);
 */

void GetCoordinates(short* x, short* y)
{
    long   a, r;
    float  angle, radius;

    a=random(31416);  /* between 0 and 10000*pi   */
    r=random(10000);

    radius = sqrt((float)r*100);
    angle  = (float)a/5000;
    *x=2000+(short)(radius*cos(angle));
    *y=2000+(short)(radius*sin(angle));
}


/*
 * this following function does the hole drift
 */

void HoleDrift(Wormhole_Struct* pHole)
{
    pHole->origx+=AVG_DRIFT-random(2*AVG_DRIFT+1);
    pHole->origy+=AVG_DRIFT-random(2*AVG_DRIFT+1);
    pHole->destx+=AVG_DRIFT-random(2*AVG_DRIFT+1);
    pHole->desty+=AVG_DRIFT-random(2*AVG_DRIFT+1);
}


/*
 * this function returns a random stability
 */
short NewStability()
{
    short s;

    s=(short)(STABILITY * tan((float)random(31416)/20000));
    return(min(s,MAX_STABILITY));
}


/*
 * this function returns a random size
 */
short NewSize()
{
    short s;

    s=(short)(BASE_SIZE  * tan((float)random(31416)/20000));
    s=min(s,MAX_SIZE);
    if (random(100)>=BIDIRECT)
       s=-s;
    return(s);
}


/*
 * this function generates a new wormhole
 */
void NewHole(Wormhole_Struct* hole)
{
    GetCoordinates(&hole->origx,&hole->origy);
    GetCoordinates(&hole->destx,&hole->desty);

    hole->stab=NewStability();
    hole->size=NewSize();
}



/*
 * this function prints the wormhole parameters
 */
void PrintHole(const char* text, Wormhole_Struct pHole, FILE* pOutputFile)
{
#ifndef __SILENT
    cprintf("\r\n%17s   %4d,%4d    %4d,%4d   %5d     %3d",text,pHole.origx,
            pHole.origy, pHole.destx, pHole.desty, pHole.size, pHole.stab);
#endif
    if (pOutputFile!=NULL)
        fprintf(pOutputFile,"\n%12s %4d,%4d    %4d,%4d   %5d     %3d",text,
                           pHole.origx, pHole.origy, pHole.destx, pHole.desty,
                           pHole.size, pHole.stab);
}



/*
 * The following routine does the random wormhole changes and prints the
 * new specifications.
 */
void CheckHoles()
{
    FILE    *lLogFile;
    short   i,
            lChanged;


    srand((unsigned short)time(NULL));

/* Here we try to improve our random generator a bit  */
    for (i=1; i<=30; i++) lChanged=random(100);

    if ((lLogFile=fopen("wormgen.log","wt"))!=NULL)
        fprintf(lLogFile,"\n                      x1,  y1      x2,  y2    size stability");


#ifndef __SILENT
    if (lLogFile==NULL)
    {
        textcolor(LIGHTRED);
        cprintf("\nunable to open wormgen.log");
    }
    textcolor(LIGHTBLUE);
    cprintf("\r\n\n                      x1,  y1      x2,  y2    size stability");

    for (i=0; i<STABLEHOLE_NR; i++)
        PrintHole("stable wormhole",gWormholes[i],lLogFile);
#endif

    for (i=STABLEHOLE_NR; i<HOLE_NR; i++)
    {
        lChanged=0;
        if (gWormholes[i].size==0)
        {
            NewHole(&gWormholes[i]);
            textcolor(LIGHTGREEN);
            PrintHole("new wormhole",gWormholes[i],lLogFile);
        }
        else
        {
            if (random(100)<CHANGE_PROB)
            {
                if (random(100)<NEW_PROB)
                {
                    NewHole(&gWormholes[i]);
                    textcolor(LIGHTGREEN);
                    PrintHole("new wormhole",gWormholes[i],lLogFile);
                }
                else
                {
                    if (random(100)<DRIFT_PROB)
                    {
                        HoleDrift(&gWormholes[i]);
                        lChanged=1;
                    }

                    if (random(100)<STAB_PROB)
                    {
                        gWormholes[i].stab=NewStability();
                        lChanged=1;
                    }

                    if (random(100)<SIZE_PROB)
                    {
                        gWormholes[i].size=NewSize();
                        lChanged=1;
                    }
                }
            }

            if (lChanged)
            {
                textcolor(MAGENTA);
                PrintHole("changed",gWormholes[i],lLogFile);
            }
            else
            {
                textcolor(LIGHTGRAY);
                PrintHole("unchanged",gWormholes[i],lLogFile);
            }
        }
    }
    if (lLogFile!=NULL) fclose(lLogFile);
}



/*
 * this function reads a wormhole.txt file, if existent.
 * If it can't read wormholes then the size is set to 0, which means that
 * CheckHoles() generates a new wormhole in this place
 */
void ReadHoles()
{
    FILE* lHoleFile;
    char  lBuffer[80];
    short lHole=0,
          lNumber;


    memset(gWormholes,0,HOLE_NR*sizeof(Wormhole_Struct));

    if ((lHoleFile=fopen("wormhole.txt","rt"))!=NULL)
    {
        while (fgets(lBuffer,80,lHoleFile)!=NULL)
            if (lBuffer[strspn(lBuffer," \t")]!=COMMENT_CHAR)
            {
                lNumber=sscanf(lBuffer,"%d %d %d %d %d %d",&gWormholes[lHole].origx,
                              &gWormholes[lHole].origy,&gWormholes[lHole].destx,&gWormholes[lHole].desty,
                              &gWormholes[lHole].size,&gWormholes[lHole].stab);
                if (lNumber!=6)                       /* read not successful  */
                    memset(&gWormholes[lHole],0,sizeof(Wormhole_Struct));
                lHole++;
            }
        fclose(lHoleFile);
    }
}



/*
 * the following routine scans the existing wormhole file and replaces
 * the wormhole specifications with the new parameters
 * Comments are copied
 * If the first line of the wormhole.txt file is NO COMMENT_CHAR (==';')
 * then at least 6 parameters (for a wormhole) are expected
 */

void WriteHoles()
{
    FILE  *lHoleFile,
          *lNewHoleFile;
    char  lBuffer[161],
          *lCommentPtr;
    short lHole=0;

    const char lCommentToken[2]={COMMENT_CHAR,0};

    lBuffer[81]=0;
    remove("wormhole.old");
    rename("wormhole.txt","wormhole.old");

    if ((lNewHoleFile=fopen("wormhole.txt","wt"))==NULL)
    {
        textcolor(LIGHTRED);
        cprintf("\r\nFatal error : can't open wormhole.txt for writing");
        textcolor(LIGHTGRAY);
        exit(1);
    }
    else
    {
/* Write the new file and copy existing file */
        lHole=0;
        if ((lHoleFile=fopen("wormhole.old","rt"))!=NULL)
        {
            while (fgets(lBuffer,160,lHoleFile)!=NULL)
            {
/*
 * If the first character is a ';' then we write the line unchanged.
 * If the first character is different then we write a wormhole and
 * append the comment in the line (if existent).
 * We do this only until we have written HOLE_NR holes, then we copy strictly
 */
                if ((lBuffer[strspn(lBuffer," \t")]==COMMENT_CHAR) || (lHole==HOLE_NR))
                    fputs(lBuffer,lNewHoleFile);
                else
                {
                    fprintf(lNewHoleFile,"%d %d %d %d %d %d",gWormholes[lHole].origx,gWormholes[lHole].origy,
                            gWormholes[lHole].destx,gWormholes[lHole].desty,gWormholes[lHole].size,gWormholes[lHole].stab);
                    lCommentPtr=strtok(lBuffer,lCommentToken);
                    lCommentPtr=strtok(NULL,lCommentToken);
                    if (strlen(lCommentPtr))
                    {
                        fprintf(lNewHoleFile," %c",COMMENT_CHAR);
                        fputs(lCommentPtr,lNewHoleFile);
                    }
                    else
                        fprintf(lNewHoleFile,"\n");
                    lHole++;
                }
            }

            fclose(lHoleFile);
        }
/* We must write the remaining holes */

        for (; lHole<HOLE_NR; lHole++)
            fprintf(lNewHoleFile,"%d %d %d %d %d %d\n",gWormholes[lHole].origx,gWormholes[lHole].origy,
                            gWormholes[lHole].destx,gWormholes[lHole].desty,gWormholes[lHole].size,gWormholes[lHole].stab);
        fclose(lNewHoleFile);
    }
}



int main(short argc, char** argv)
{
    char lOldpath[MAXDIR];

    gWormholes=(Wormhole_Struct*)calloc(HOLE_NR,sizeof(Wormhole_Struct));

    if (argc>1)
    {
        getcwd(lOldpath,MAXDIR);
        if (chdir(argv[1]))
        {
            printf("\nFatal error : Can't chdir to %s",argv[1]);
            return 1;
        }
        else
        {
            ReadHoles();
            CheckHoles();
            WriteHoles();
            if (chdir(lOldpath))
                printf("\nFatal error : Can't chdir back to %s",argv[1]);
            return 1;
        }
    }
    else
    {
        ReadHoles();
        CheckHoles();
        WriteHoles();
    }

    return 0;
}


