#include "wormgen.h"
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <mem.h>
#include <dir.h>


#define sqr(x)      ((x)*(x))
#define random(num) ((short)((long)rand()*(num)/(RAND_MAX+1)))
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))



/*
   the following function generates a random x,y position on your map
   change this function according to your galaxy

   the standard galaxy would be
   *x=1000+random(2000);
   *y=1000+random(2000);
*/

void getcoordinates(short* x, short* y)
{
/* this is for a circular galaxy with a 1000 ly radius around 2000,2000  */
    long   a, r;
    float  angle, radius;

    a=random(31416);  /* between 0 and 1000*pi   */
    r=random(10000);

    radius=sqrt((float)r*100);
    *x=2000+(short)(radius*cos((float)a/5000));
    *y=2000+(short)(radius*sin((float)a/5000));
}




void holedrift(wormhole* hole)
{
    hole->origx+=avgdrift-random(2*avgdrift+1);
    hole->origy+=avgdrift-random(2*avgdrift+1);
    hole->destx+=avgdrift-random(2*avgdrift+1);
    hole->desty+=avgdrift-random(2*avgdrift+1);
}

short newstability()
{
    short s;

    s=(short)(stability * tan((float)random(31416)/20000));
    return(min(s,maxstability));
}

short newsize()
{
    short s;

    s=(short)(basesize  * tan((float)random(31416)/20000));
    s=min(s,maxsize);
    if (random(100)>=bidirect)
       s=-s;
    return(s);
}

/*
   generate a new wormhole
*/
void newhole(wormhole* hole)
{
    getcoordinates(&hole->origx,&hole->origy);
    getcoordinates(&hole->destx,&hole->desty);

    hole->stab=newstability();
    hole->size=newsize();
}




void printhole(const char* text, wormhole hole, FILE* outputfile)
{
    cprintf("\r\n%12s   %4d,%4d    %4d,%4d   %5d     %3d",text,hole.origx,
            hole.origy, hole.destx, hole.desty, hole.size, hole.stab);

    if (outputfile!=NULL)
        fprintf(outputfile,"\n%12s %4d,%4d    %4d,%4d   %5d     %3d",text,
                           hole.origx, hole.origy, hole.destx, hole.desty,
                           hole.size, hole.stab);
}


void checkholes()
{
    FILE    *f;
    short   i,
            changed;


    srand((unsigned short) time(NULL));

    for (i=1; i<=30; i++) changed=random(100);

    if ((f=fopen("wormgen.log","wt"))==NULL)
    {
        textcolor(LIGHTRED);
        cprintf("\nunable to open wormgen.log");
    }
    else
        fprintf(f,"\n                 x1,  y1      x2,  y2    size stability");


    textcolor(LIGHTBLUE);
    cprintf("\r\n\n                 x1,  y1      x2,  y2    size stability");
    for (i=0; i<holenr; i++)
    {
        changed=0;
        if (holes[i].size==0)
        {
            newhole(&holes[i]);
            textcolor(LIGHTGREEN);
            printhole("new wormhole",holes[i],f);
        }
        else
        {
            if (random(100)<changeprob)
            {
                if (random(100)<newprob)
                {
                    newhole(&holes[i]);
                    textcolor(LIGHTGREEN);
                    printhole("new wormhole",holes[i],f);
                }
                else
                {
                    if (random(100)<driftprob)
                    {
                        holedrift(&holes[i]);
                        changed=1;
                    }

                    if (random(100)<stabprob)
                    {
                        holes[i].stab=newstability();
                        changed=1;
                    }

                    if (random(100)<sizeprob)
                    {
                        holes[i].size=newsize();
                        changed=1;
                    }
                }
            }

            if (changed)
            {
                textcolor(MAGENTA);
                printhole("changed",holes[i],f);
            }
            else
            {
                textcolor(LIGHTGRAY);
                printhole("unchanged",holes[i],f);
            }
        }
    }
    if (f!=NULL) fclose(f);
}




void readholes()
{
    FILE* holefile;
    char  buffer[80];
    short hole=0,
          number;

    memset(holes,0,holenr*wormholesize);

    if ((holefile=fopen("wormhole.txt","rt"))!=NULL)
    {
        while (fgets(buffer,80,holefile)!=NULL)
            if (buffer[0]!=commentchar)
            {
                number=sscanf(buffer,"%d %d %d %d %d %d",&holes[hole].origx,
                              &holes[hole].origy,&holes[hole].destx,&holes[hole].desty,
                              &holes[hole].size,&holes[hole].stab);
                if (number!=6)                       /* read not successful  */
                    memset(&holes[hole],0,wormholesize);
                hole++;
            }
        fclose(holefile);
    }
}


void writeholes()
{
    FILE  *holefile,
          *newfile;
    char  buffer[81];
    short hole=0;


    buffer[81]=0;
    remove("wormhole.old");
    rename("wormhole.txt","wormhole.old");

    if ((newfile=fopen("wormhole.txt","wt"))==NULL)
    {
        textcolor(LIGHTRED);
        cprintf("\r\nFatal error : can't open wormhole.txt for writing");
        textcolor(LIGHTGRAY);
    }
    else
    {
        if ((holefile=fopen("wormhole.old","rt"))!=NULL)   /* copy comments */
        {
            while (fgets(buffer,80,holefile)!=NULL)
                if (buffer[0]==commentchar)
                    fputs(buffer,newfile);
            fclose(holefile);
        }

        for (hole=0; hole<holenr; hole++)
            fprintf(newfile,"%d %d %d %d %d %d\n",holes[hole].origx,holes[hole].origy,
                            holes[hole].destx,holes[hole].desty,holes[hole].size,holes[hole].stab);
        fclose(newfile);
    }
}


main(short argc, char** argv)
{
    char oldpath[MAXDIR];


    if (argc>1)
    {
        getcwd(oldpath,MAXDIR);
        if (chdir(argv[1]))
            printf("\nFatal error : Can't chdir to %s",argv[1]);
        else
        {
            readholes();
            checkholes();
            writeholes();
            if (chdir(oldpath))
                printf("\nFatal error : Can't chdir to %s",argv[1]);
        }
    }
    else
    {
        readholes();
        checkholes();
        writeholes();
    }

    return(0);
}


