/* base options */
static const short HOLE_NR      =  15;  /* number of wormholes  */
static const short STABLEHOLE_NR=   5;  /* the first 5 won't be changed */

static const short BASE_SIZE    = 250;  /* base size for mass                     */
static const short MAX_SIZE     =3000;  /* maximum size                           */
static const short STABILITY    =  10;  /* base stability                         */
static const short MAX_STABILITY= 100;  /* max stability                          */
static const short AVG_DRIFT    =  10;  /* avg drift for every coordinate         */
static const short BIDIRECT     =  65;  /* 65% of the holes are bidirectional     */

/*
 * Note : Every hole has a changeprob chance to get checked.
 * Only checked holes can change. If a hole is checked then it has a
 * newprob chance of collapsing and reappearing as different hole,
 * so the total chance of a collaps would be .20 * .15 = .03 = 3%
 * If a hole doesn't collaps then all other change possibilities are checked
 * independent of each other.
 * The probabilities for drifts, size and stability changes work similar.
 */

/* change options */
static const short CHANGE_PROB  =  20;  /* probability for a wormhole check (%)   */
static const short NEW_PROB     =  15;  /* probability for a completely new hole  */
static const short DRIFT_PROB   =  15;  /* probability to drift                   */
static const short STAB_PROB    =  15;  /* probability for a stability change     */
static const short SIZE_PROB    =  15;  /* probability for a size change          */


/* define this option to supress output to screen */

#undef __SILENT


/* some other constants   */
#define COMMENT_CHAR ';'    /* ";", lines starting with this constant are comments  */


typedef struct
{
    short    origx,
             origy,
             destx,
             desty,
             size,
             stab;
}
Wormhole_Struct;


Wormhole_Struct *gWormholes;


void GetCoordinates(short* x, short* y);

void HoleDrift(Wormhole_Struct* pHole);

short NewStability(void);

short NewSize(void);

void NewHole(Wormhole_Struct* hole);

void PrintHole(const char* text, Wormhole_Struct pHole, FILE* pOutputFile);

void CheckHoles(void);

void ReadHoles(void);

void WriteHoles(void);
